#include <LEDA/integer.h>
#include <LEDA/integer_old.h>
#include <LEDA/stream.h>

#if defined(LEDA_STD_IO_HEADERS)
using namespace std;
#endif



template <class integer_t> 
void integer_test(const integer_t& A, const integer_t& B, int n)
{
   int i;

   int n1 = 1000*n;

   int steps = n/10;

   if (steps == 0) steps = n;

   integer_t D,P,Q,R,S;

   float T = used_time();

   cout << string("%8d additions        ",n1) << flush;
   T = used_time();
   for(i=0; i < n; i++) 
   { if (i%steps == 0) cout << "." << flush;
     for(int j=0; j < 1000; j++) S = A+B;
    }
   cout << string(" %4.2f",used_time(T)) << endl;

   cout << string("%8d subtractions     ",n1) << flush;
   T = used_time();
   for(i=0; i < n; i++) 
   { if (i%steps == 0) cout << "." << flush;
     for(int j=0; j < 1000; j++) D = A-B;
    }
   cout << string(" %4.2f",used_time(T)) << endl;

   if (S+D-A != A) 
     cout << "error in addition/subtraction" << endl;

   cout << string("%8d multiplications  ",n) << flush;
   T = used_time();
   for(i=0; i<n; i++) 
   { if (i%steps == 0) cout << "." << flush;
     P = A*B; 
    }
   cout << string(" %4.2f",used_time(T)) << endl;


   cout << string("%8d divisions        ",n) << flush;
   for(i=0; i < n; i++)
   { if (i%steps == 0) cout << "." << flush;
     Q = A/B; 
    }
   cout << string(" %4.2f",used_time(T)) << endl;


   cout << string("%8d modulos          ",n) << flush;
   for(i=0; i < n; i++)
   { if (i%steps == 0) cout << "." << flush;
     R = A%B; 
    }
   cout << string(" %4.2f",used_time(T)) << endl;

   if (Q*B + R != A) 
      cout << "error in division/multiplication" << endl;

}



int main()
{

  int la = read_int("#words of A  = ");
  int lb = read_int("#words of B  = ");
  int n  = read_int("iterations   = ");

  la = la/(sizeof(unsigned long)/4);
  lb = lb/(sizeof(unsigned long)/4);

  unsigned int* a = new unsigned int[la];
  unsigned int* b = new unsigned int[lb];

  random_source ran;

  int i;
  for(i=0; i<la; i++) a[i] = ran();
  for(i=0; i<lb; i++) b[i] = ran();

  newline;
  cout << "LEDA integers" << endl;
  integer A = 0;
  for(i=0; i<la; i++) A = (A << 32) + a[i];
  integer B = 0;
  for(i=0; i<lb; i++) B = (B << 32) + b[i];
  integer_test(A,B,n);
  newline;

  newline;
  cout << "Old  integers" << endl;
  integer_old A1 = 0;
  for(i=0; i<la; i++) A1 = (A1 << 32) + a[i];
  integer_old B1 = 0;
  for(i=0; i<lb; i++) B1 = (B1 << 32) + b[i];
  integer_test(A1,B1,n);
  newline;

  delete[] a;
  delete[] b;

  return 0;

 }
